const TaskQueue = require('@goodware/task-queue');

{
const queue = new TaskQueue({ size: 2 });

// Task #1 : await push() returns immediately because the queue is empty. 'await' doesn't wait for the 
// task to complete.
await queue.push(
  () =>
    new Promise((resolve) =>
      setTimeout(() => {
        console.log(`Task 1 ${Date.now()}`);
        resolve();
      }, 400)
    )
);

// Task #2 : await push() returns immediately because the queue has an open slot
await queue.push(
  () =>
    new Promise((resolve) =>
      setTimeout(() => {
        console.log(`Task 2 ${Date.now()}`);
        resolve();
      }, 300)
    )
);

// The queue is full. Task #2 will finish in about 300 ms.

// Task #3 : await push() waits until task #2 finishes
await queue.push(
  () =>
    new Promise((resolve) =>
      setTimeout(() => {
        console.log(`Task 3 ${Date.now()}`);
        resolve();
      }, 200)
    )
);

// The queue is full again. 300 ms have already passed. Task #1 will terminate in about 100 ms, leaving 
// task #3 in the queue.

// Task #4 : await push() waits until task #1 finishes
const ret = await queue.push(
  () =>
    new Promise((resolve) =>
      setTimeout(() => {
        console.log(`Task 4 ${Date.now()}`);
        resolve();
      }, 100)
    )
);

// Wait for task #4 to finish
await ret.promise;
await queue.stop();
};

{
const queue = new TaskQueue({ size: 5, workers: 2 });
let tasks = 0;

// Wait 10 ms
async function doWork() {
  const me = ++tasks;
  console.log(`${me} begin`);
  await new Promise((resolve) => setTimeout(resolve, 10));
  console.log(`${me} end`);
}

// This will run fewer than 25 tasks. The actual number is random.
for (let i = 1; i <= 25; ++i) {
  if (queue.full) {
    // The most basic implementation of backpressure: do nothing
    // Another interesting form of backpressure would be to call push()
    // infrequently. For example...
    if ((Math.random()*8) < 2) {
      console.log('infrequently queued');
      await queue.push(doWork);
    } else {
      console.log('full');
    }
  }  
  else {
    console.log('queued');
    await queue.push(doWork);
  }    
}

await queue.stop();
console.log(`Ran ${tasks} tasks`);
};


{
const queue = new TaskQueue({ size: 1 });

// Task #1 : await push() returns immediately because the queue is empty. 'await' doesn't wait for the 
// task to complete.
await queue.push(
  () =>
    new Promise((resolve) =>
      setTimeout(() => {
        console.log(`Task 1 ${new Date().toISOString()}`);
        resolve();
      }, 400)
    )
);

// The following calls to push() return Promises. The provided
// functions are not queued until there's an available slot in the 
// queue. The are queued in the order in which push() is called.

// Task #2 (background) : await push() waits until task #1 finishes
queue.push(
  () =>
    new Promise((resolve) =>
      setTimeout(() => {
        console.log(`Task 2 ${new Date().toISOString()}`);
        resolve();
      }, 2000)
    )
);

// Task #3 (background) : await push() waits until task #2 finishes
queue.push(
  () =>
    new Promise((resolve) =>
      setTimeout(() => {
        console.log(`Task 3 ${new Date().toISOString()}`);
        resolve();
      }, 1000)
    )
);

// Task #4 (background) : await push() waits until task #2 finishes
queue.push(
  () =>
    new Promise((resolve) =>
      setTimeout(() => {
        console.log(`Task 4 ${new Date().toISOString()}`);
        resolve();
      }, 5)
    )
);

await queue.wait();
console.log('stopped');
};
